-- ---------------------------------------------------------------------------------------------------------------------------------
---
--		Thanks to:
--
--		Yata:
--		To study the code of Yata helped me a lot to realize this mod.
--
--		Numen:
--		Without Numen I would not have ever made the rework and improvments of Klappa.
--		Took a lot of code.
--
-- --------------------------------------------------------------------------------------------------------------------------------

Klappa = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDB-2.0", "FuBarPlugin-2.0", "AceConsole-2.0", "AceDebug-2.0");
local L = AceLibrary("AceLocale-2.2"):new("Klappa")

local _G = getfenv(0)

-- Fubar-Options
Klappa.name = "Klappa";
Klappa.hasIcon = "Interface\\Icons\\Inv_Misc_SummerFest_BrazierRed";
Klappa.hasNoColor = true;
Klappa.defaultMinimapPosition = 200;
Klappa.clickableTooltip  = true;
Klappa.independentProfile = true;
Klappa.defaultPosition = "RIGHT";

Klappa.isLocked = true;

local defaults = 
{
	buttons = {},
	numberButtons = 2,
	orient = "vertleft",
	buttonScale = 1,
	popupScale = 1,
	alpha = 1,
	popupAlpha = 1,
	locked = true,
	padding = 1,
}

Klappa:RegisterDB("KlappaDB", "KlappaDBPC");
Klappa:RegisterDefaults("profile", defaults);

		

function Klappa:OnInitialize()
	
	local short, long = self:GetProfile();
	if (short == "Default") then
		self:Print("Char based Profiles are default now");
		self:SetProfile("char");
	end
	self:RegisterChatCommand("/klappa", self.options);
	--Klappa.config = self.db.profile;
	self.options = self:InitOptions();
	self.OnMenuRequest = self.options;
end

function Klappa:OnEnable()
	self:SetDebugging(false);
	self:Debug("Enabled");
	self:CreateRoot();
	self:LoadPosition();
	self:UpdateLayout();
	self:LoadSpells();
end

function Klappa:OnDisable()

end

function Klappa:CreateRoot()

	self.root = CreateFrame("Frame", "KlappaRoot", UIParent, "SecureStateHeaderTemplate");
	self.root:SetPoint("CENTER", UIParent, "CENTER", 0, 0);
	self.root:RegisterForDrag("LeftButton");
	self.root:EnableMouse(false);
	self.root:SetMovable(true);
	self.root:SetClampedToScreen(true);
	
	self.root.texture = Klappa.root:CreateTexture();
	self.root.texture:SetTexture(0,0,0.5,0);
	self.root.texture:SetAllPoints(self.root);
	
	self.root.buttons = {};
	for i = 1, self.db.profile.numberButtons do
		self.root.buttons[i] = Klappa.Button:new(i,"bar"..i);
	end
	
	self:Debug("Button loaded");
end

function Klappa:UpdateLayout()
	local rootx, rooty = 0, 0;
	local isVert, isRtDn = false, false;
	if (self.db.profile.orient == "horzdown") then
		isRtDn = true;
	elseif (self.db.profile.orient == "vertleft") then
		isVert = true;
	elseif (self.db.profile.orient == "vertright") then
		isVert = true;
		isRtDn = true;
	end
	if (isVert) then
		rootx = 36;
	else
		rooty = 36;
	end
	for idx, buttonClass in pairs(self.root.buttons) do
		local x, y = 0, 0;
		local padding = Klappa.db.profile.padding;
		if (isVert) then
			y = (-36-padding)*(buttonClass.index - 1);
			rooty = rooty + 36 + padding;
		else
			x = (36+padding)*(buttonClass.index - 1);
			rootx = rootx + 36 + padding;
		end
		buttonClass:UpdateLayout(x, y, isVert, isRtDn);
	end
	self.root:SetScale(Klappa.db.profile.buttonScale);
	self.root:SetWidth(rootx);
	self.root:SetHeight(rooty);
end

function Klappa:ToggleLock()
	if (self.isLocked) then
		self.root:EnableMouse(true);
		self.root:SetScript("OnDragStart", function() Klappa:StartDrag(); end);
		self.root:SetScript("OnDragStop", function() Klappa:StopDrag(); end);
		self.root.texture:SetTexture(0, 0, 0.5, 0.5)
		self.root:SetFrameLevel(4);
		self.isLocked = false;
		-- Show all popups
		for idx, buttonClass in pairs(self.root.buttons) do
			buttonClass:ShowPops();
		end
	else
		self.root:EnableMouse(false);
		self.root:SetScript("OnDragStart", nil);
		self.root:SetScript("OnDragStop", nil);
		self.root.texture:SetTexture(0, 0, 0.5, 0);
		self.isLocked = true;
	end		
end

function Klappa:StartDrag()
	Klappa.root:StartMoving();
end

function Klappa:StopDrag()
	Klappa.root:StopMovingOrSizing();
	Klappa:SavePosition();
end

function Klappa:LoadPosition()
	local x, y, s = self.db.profile.rootX, self.db.profile.rootY, self.root:GetEffectiveScale();
	if x and y then 
		x, y = x/s, y/s;
	end
	self.root:ClearAllPoints();
	self.root:SetPoint(x and "TOPLEFT" or "CENTER", UIParent, x and "BOTTOMLEFT" or "CENTER", x or 0, y or 0);
end

function Klappa:SavePosition()
	local x, y, s = self.root:GetLeft(), self.root:GetTop(), self.root:GetEffectiveScale();
	x, y = x*s, y*s;
	self.db.profile.rootX = x;
	self.db.profile.rootY = y;
end

function Klappa:LoadSpells()
	Klappa:Debug("Load Spells:");
	for idx, buttonClass in pairs(Klappa.root.buttons) do
		if(self.db.profile.buttons[idx] ~="" and self.db.profile.buttons[idx] ~= nil) then
			Klappa:Debug("index: "..idx.." :: ");
			if(self.db.profile.buttons and self.db.profile.buttons[idx]) then
				buttonClass.spellName = Klappa.db.profile.buttons[idx].spellName;
				buttonClass.spell = Klappa.db.profile.buttons[idx].spell;
				buttonClass.button.type = Klappa.db.profile.buttons[idx].spelltype;
				buttonClass.button.spellID = Klappa.db.profile.buttons[idx].spellID;
				if(buttonClass.spell ~= nil) then
					Klappa:Debug("Spell:  "..buttonClass.spell);	
					buttonClass.button.texture:SetTexture(Klappa.db.profile.buttons[idx].texture);
					if(buttonClass.button.type ~= nil) then
						Klappa:Debug(buttonClass.button.type);
						buttonClass.button:SetAttribute("*type*", buttonClass.button.type);
						buttonClass.button:SetAttribute(buttonClass.button.type.."1", buttonClass.spell);
					end
				end
			end
		else
			return;
		end
		for idx2, popupbuttonClass in pairs(Klappa.root.buttons[idx].popupButtons) do
			if(self.db.profile.buttons[idx].popupButtons ~="" and self.db.profile.buttons[idx].popupButtons ~= nil) then
				Klappa:Debug("PopUpindex: "..idx2.." :: ");
				if(Klappa.db.profile.buttons[idx].popupButtons and Klappa.db.profile.buttons[idx].popupButtons[idx2]) then
					popupbuttonClass.spellName = Klappa.db.profile.buttons[idx].popupButtons[idx2].spellName;
					popupbuttonClass.spell = Klappa.db.profile.buttons[idx].popupButtons[idx2].spell;
					popupbuttonClass.button.type = Klappa.db.profile.buttons[idx].popupButtons[idx2].spelltype;
					popupbuttonClass.button.spellID = Klappa.db.profile.buttons[idx].popupButtons[idx2].spellID;
					if(popupbuttonClass.spell ~= nil) then
						Klappa:Debug("PopUpSpell:  "..popupbuttonClass.spell);	
						popupbuttonClass.button.texture:SetTexture(Klappa.db.profile.buttons[idx].popupButtons[idx2].texture);
						if(popupbuttonClass.button.type ~= nil) then
							Klappa:Debug(popupbuttonClass.button.type);
							popupbuttonClass.button:SetAttribute("*type*", popupbuttonClass.button.type);
							popupbuttonClass.button:SetAttribute(popupbuttonClass.button.type.."1", popupbuttonClass.spell);
						end
					end
				end
			end
		end		
	end
	Klappa:Debug("Spells loaded");
end

function Klappa:ShowTooltip(button)
	if (button.spellID) then
		Klappa:Debug(button.spellID.." :: "..button.type)
		if(button.type == "spell") then
			GameTooltip:SetOwner(button);
			GameTooltip:SetSpell(button.spellID, BOOKTYPE_SPELL);
		elseif(button.type == "item") then
			GameTooltip:SetOwner(button);
			local spellName, sLink, iRarity, iLevel, iMinLevel, sType, sSubType, iStackCount, itemEquipLoc, texture = GetItemInfo(button.spellID);
			GameTooltip:SetHyperlink(sLink);
		elseif(button.type == "macro") then
			GameTooltip:SetOwner(button);
			name, texture, body, localvar = GetMacroInfo(button.spellID);
			GameTooltip:SetText(name);
		end
	end
end

function Klappa:HideTooltip()
	GameTooltip:Hide();
end

function Klappa:AddMainButton()
	idx = self.db.profile.numberButtons + 1;
	self.root.buttons[idx] = Klappa.Button:new(idx,"bar"..idx);
	self.db.profile.numberButtons = idx;
	self:UpdateLayout();
end

function Klappa:DelMainButton()
	index = self.db.profile.numberButtons;
	if (index == 0 or index == nil) then return end;
	self.root.buttons[index].button:Hide();
	self.root.buttons[index].button.popups = nil;
	self.root.buttons[index].button = nil;
	self.root.buttons[index] = nil;
	
	--if(Klappa.db.profile.buttons[index].popups ~= nil) then
	--	Klappa.db.profile.buttons[index].popups = nil;
	--end
	Klappa.db.profile.buttons[index] = nil;
	Klappa.options.args["Main"..index] = nil;
	self.db.profile.numberButtons = index-1;
	self:UpdateLayout();
end
