if not (Klappa) then return; end;
local _G = getfenv(0);

local Klappa = Klappa;
local AceOO = AceLibrary("AceOO-2.0");
local L = AceLibrary("AceLocale-2.2"):new("Klappa");

Klappa.Button = AceOO.Class("AceEvent-2.0");


function Klappa.Button.prototype:init(idx,type)
	Klappa.Button.super.prototype.init(self);
	self.index = idx;
	self.header = CreateFrame("Frame", "Klappa"..type.."Frame", Klappa.root, "SecureStateHeaderTemplate");
	self.button = CreateFrame("CheckButton", "Klappa"..type.."Button", self.header, "SecureAnchorEnterTemplate,SecureActionButtonTemplate,ActionButtonTemplate");
	self.button.class = self;
	if(Klappa.db.profile.buttons[idx] ~= nil and Klappa.db.profile.buttons[idx].popups ~= nil ) then
		self.button.popups = Klappa.db.profile.buttons[idx].popups;
	else
		Klappa:Debug("No saved popups");
		self.button.popups = 1;
		if(Klappa.db.profile.buttons == nil) then
			Klappa.db.profile.buttons = {};
		end
		Klappa.db.profile.buttons[idx] = {};
		Klappa.db.profile.buttons[idx].popups = self.button.popups;
	end
	self.button:SetPoint("TOPLEFT", self.header, "TOPLEFT", 0, 0);
	self.button:SetWidth(36);
	self.button:SetHeight(36);

	self.button:SetAttribute("type", "spell");
	self.button:SetAttribute("spell", "");
	self.button:SetAttribute("childstate-onenter", "show"..type);
	self.button:SetAttribute("childstate-onleave", "hide"..type);
	self.button:SetAttribute("anchorchild", Klappa.root);

	self.button:HookScript("OnEnter", function(b) Klappa:ShowTooltip(b); end);
	self.button:HookScript("OnLeave", function() Klappa:HideTooltip(); end);
	self.button:SetScript("OnDragStart", function() Klappa.root.buttons[idx]:ButtonDrag(); end);
	self.button:SetScript("OnReceiveDrag", function() Klappa.root.buttons[idx]:ButtonDrop(); end);
	self.button:RegisterForDrag("LeftButton");
	
	self.button.texture = self.button:CreateTexture(nil, "BACKGROUND");
	self.button:SetCheckedTexture("");
	self.button:SetNormalTexture("");
	self:ApplyStyle();
	self.button.texture:SetPoint("TOPLEFT", self.button, "TOPLEFT", 3, -3);
	self.button.texture:SetPoint("BOTTOMRIGHT", self.button, "BOTTOMRIGHT",  -3, 3);
	self.button.texture:SetTexCoord(0.08, 0.92, 0.08, 0.92);
	self.header:SetAttribute("addchild", self.button);
	Klappa.root:SetAttribute("statemap-anchor-show"..type, "*:"..idx);
	Klappa.root:SetAttribute("statemap-anchor-hide"..type, ";");
	Klappa.root:SetAttribute("delaystatemap-anchor-hide"..type, idx..":0");
	Klappa.root:SetAttribute("delaytimemap-anchor-hide"..type,  idx..":0.5");
	Klappa.root:SetAttribute("delayhovermap-anchor-hide"..type, idx..":true");
	
	--Klappa.db.profile.buttons[self.index] = {}
	--Klappa.db.profile.buttons[self.index].popupButtons = {};
	
	self.popupButtons = {};
	for i=1,self.button.popups or 1 do
		self.popupButtons[i] = Klappa.PopUpButton:new(self,i,type);
	end
	self.button.popupButtons = self.popupButtons;
	self.button.index = self.index;
	
	self.button.cooldown = _G[self.button:GetName().."Cooldown"];
	self:RegisterEvent("ACTIONBAR_UPDATE_COOLDOWN", "ButtonEventHandler");
	self:AddOptions();
	Klappa:Debug("button created");
end

function Klappa.Button.prototype:UpdateLayout(x, y, isVert, isRtDn)
	self.header:ClearAllPoints();
	self.header:SetPoint("TOPLEFT", Klappa.root, "TOPLEFT", x, y);
	local scale = Klappa.db.profile.buttonScale;
	for idx, popButtonClass in pairs(self.popupButtons) do
		local x, y = 0, 0;
		local index = idx;
		local padding = Klappa.db.profile.padding;
		if (isVert) then
			x = (36+padding)*index - 36;
			if (not isRtDn) then
				x = x*(-1);
			end
		else
			y = (-36-padding)*index + 36;
			if (not isRtDn) then
				y = y*(-1);
			end
		end
		popButtonClass:UpdateLayout(isVert, isRtDn, x/scale, y/scale);
	end
	self.button:SetAlpha(Klappa.db.profile.alpha);
end

function Klappa.Button.prototype:ShowPops()
	for idx, popupButtonClass in pairs (self.popupButtons) do
		popupButtonClass.button:Show();
	end
end

function Klappa.Button.prototype:ApplyStyle()
	local header = CreateFrame("Frame", self.button:GetName().."DL", self.button);
	header:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 1, edgeFile = "", edgeSize = 0, insets = {left = 0, right = 0, top = 0, bottom = 0},})
	header:SetBackdropColor(0, 0, 0, 0.6)
	header:SetAllPoints(self.button)
	header:SetFrameLevel(0)
end

function Klappa.Button.prototype:AddOptions()
	Klappa.options.args["Main"..self.index] = {
		name = L["Main button "]..self.index,
		desc = L["Options for the button"],
		type = "group",
		args = {}
		};
	Klappa.options.args["Main"..self.index].args = {
		options = {
			name = L["Add popup button"],
			desc = L["Add a new popup button"],
			type = "execute",
			order = 1,
			func = function() Klappa.root.buttons[self.index]:AddPopup() end
		}, 
		delPopup = {
			name = L["Delete popup button"],
			desc = L["Delete the last popup button"],
			type = "execute",
			order = 2,
			func = function() Klappa.root.buttons[self.index]:DelPopup() end
		}
	}
end

function Klappa.Button.prototype:AddPopup()
	self.button.popups = self.button.popups + 1;
	self.button.popupButtons[self.button.popups] = Klappa.PopUpButton:new(self,self.button.popups,"bar"..self.index);
	Klappa.db.profile.buttons[self.index].popups = self.button.popups;
	Klappa:UpdateLayout();
end

function Klappa.Button.prototype:DelPopup()
	if(self.button.popups <= 0) then self.button.popups = 0; return end;
	self.button.popupButtons[self.button.popups].button.texture:Hide();
	self.button.popupButtons[self.button.popups].button:SetAttribute("showstates", "!*");
	self.button.popupButtons[self.button.popups].button:Hide();
	self.button.popupButtons[self.button.popups].button = nil;
	self.button.popupButtons[self.button.popups] = nil;
	self.button.popups = self.button.popups - 1;
	Klappa.db.profile.buttons[self.index].popups = self.button.popups;
	Klappa:UpdateLayout();
end


function Klappa.Button.prototype:ButtonDrag()
	Klappa:Debug("drag");
	Klappa:Debug(self.spell);
	Klappa:Debug(self.spellName);
	if(self.spell ~= "") then
		if(self.button.type == "spell") then
			Klappa:Debug("Drag ID");
			Klappa:Debug(self.spell);
			PickupSpell(self.spell);
			self.button:SetAttribute("spell1", "");
			self.spell = "";
			self.button.spellID = nil;
			self.button.texture:SetTexture("");
		elseif(self.button.type == "macro") then
			PickupMacro(self.spell);
			self.button:SetAttribute("spell1", "");
			self.button.spell = "";
			self.button.spellID = nil;
			self.button.texture:SetTexture("");
		else
			PickupSpell(self.spell);
			self.button:SetAttribute("spell1", "");
			self.button.spell = "";
			self.button.spellID = nil;
			self.button.texture:SetTexture("");
		end
	end
end

function Klappa.Button.prototype:ButtonDrop()
	Klappa:Debug("drop");
	spelltype, slot = GetCursorInfo()
	self.button.spellID = slot;
	self.button.type = spelltype;
	if(spelltype == nil or slot == nil) then return end;
	
	Klappa:Debug("Spell:"..spelltype.."  slot: "..slot);
	Klappa.db.profile.buttons[self.index] = {};
	local curtexture = "";
	self.button:SetAttribute("*type*", spelltype);
	if(CursorHasSpell()) then
		local spellName, spellRank = GetSpellName(slot, BOOKTYPE_SPELL)
		self.spellName = spellName;
		self.spell = spellName.."("..spellRank..")";
		curtexture = GetSpellTexture(slot,BOOKTYPE_SPELL);
		self.button.texture:SetTexture(curtexture);
		self.button:SetAttribute(spelltype.."1", spellName.."("..spellRank..")");
		Klappa.db.profile.buttons[self.index].spell = spellName.."("..spellRank..")";
		Klappa.db.profile.buttons[self.index].texture = curtexture;
		Klappa.db.profile.buttons[self.index].spellID = slot;
		Klappa.db.profile.buttons[self.index].spellName = spellName;
		Klappa.db.profile.buttons[self.index].spelltype = spelltype;
		--Klappa.db.profile.buttons[self.index].texture = self.button.texture:GetTexture();
		ClearCursor();
		Klappa:Debug("ist Spell");
	elseif(CursorHasItem()) then
		local spellName, sLink, iRarity, iLevel, iMinLevel, sType, sSubType, iStackCount, itemEquipLoc, texture = GetItemInfo(slot);
		self.button:SetAttribute(spelltype.."1", spellName);
		self.spell = spellName;
		self.spellName = spellName;
		self.button.texture:SetTexture(texture);
		curtexture = texture;
		Klappa.db.profile.buttons[self.index].spell = spellName;
		Klappa.db.profile.buttons[self.index].texture = curtexture;
		Klappa.db.profile.buttons[self.index].spellID = slot;
		Klappa.db.profile.buttons[self.index].spellName = spellName;
		Klappa.db.profile.buttons[self.index].spelltype = spelltype;
		--Klappa.db.profile.buttons[self.index].texture = self.button.texture:GetTexture();
		ClearCursor();
	elseif(CursorHasMacro()) then
		local spellName, texture, macrobody, localVar = GetMacroInfo(slot)
		self.button:SetAttribute(spelltype.."1", spellName);
		self.spell = spellName;
		self.spellName = spellName;
		self.button.texture:SetTexture(texture);
		curtexture = texture;
		Klappa.db.profile.buttons[self.index].spell = spellName;
		Klappa.db.profile.buttons[self.index].texture = curtexture;
		Klappa.db.profile.buttons[self.index].spellID = slot;
		Klappa.db.profile.buttons[self.index].spellName = spellName;
		Klappa.db.profile.buttons[self.index].spelltype = spelltype;
		--Klappa.db.profile.buttons[self.index].texture = self.button.texture:GetTexture();
		ClearCursor();
	end
end

function Klappa.Button.prototype:ButtonEventHandler(e)
	if (event == "ACTIONBAR_UPDATE_COOLDOWN") then
		self:UpdateCooldown();
	end
end

function Klappa.Button.prototype:UpdateCooldown()
	if (self.button.spellID and self.button.cooldown) then
		Klappa:Debug(self.button.type);
		--Klappa:Debug(self.button.spellID);
		if(self.button.type == "spell") then
			local start, duration, enable = GetSpellCooldown(self.button.spellID, BOOKTYPE_SPELL);
			CooldownFrame_SetTimer(self.button.cooldown, start, duration, enable)
		elseif(self.button.type == "item") then
			local start, duration, enable = GetItemCooldown(self.button.spellID);
			Klappa:Debug("Start:");
			Klappa:Debug(start);
			Klappa:Debug(duration);
			Klappa:Debug(enable);
			CooldownFrame_SetTimer(self.button.cooldown, start, duration, enable)
		end	
	end
end
